'use strict';

/**
 * Sigma.js Labels Heuristics
 * ===========================
 *
 * Miscelleneous heuristics related to label display.
 */

/**
 * Constants.
 */

// Dimensions of a normal cell
var DEFAULT_CELL = {
  width: 200,
  height: 150
};

// Dimensions of an unzoomed cell. This one is usually smaller than the normal
// one to account for the fact that labels will more likely collide.
var DEFAULT_UNZOOMED_CELL = {
  width: 400,
  height: 300
};

// TODO: use degree as secondary tie-breaker

/**
 * Label grid heuristic selecting labels to display according to the following
 * parameters:
 *   1) Only one label per grid cell.
 *   2) Greater labels win.
 *   3) Already displayed label should stay displayed.
 *
 * Note: It is possible to apply a size threshold to the labels (but should
 * really be done at quad level for performance).
 *
 * Note: It might be possible to not use last displayed labels by measurements
 * and a margin.
 *
 * @param  {object} params                 - Parameters:
 * @param  {object}   cache                - Cache storing nodes' data.
 * @param  {Camera}   camera               - The renderer's camera.
 * @param  {Set}      displayedLabels      - Currently displayed labels.
 * @param  {Set}      previousVisibleNodes - Nodes visible last render.
 * @param  {Array}    visibleNodes         - Nodes visible for this render.
 * @return {Array}                         - The selected labels.
 */
exports.labelsToDisplayFromGrid = function (params) {
  var cache = params.cache,
      camera = params.camera,
      displayedLabels = params.displayedLabels,
      previousVisibleNodes = params.previousVisibleNodes,
      visibleNodes = params.visibleNodes;


  var cameraState = camera.getState(),
      previousCameraState = camera.getPreviousState(),
      dimensions = camera.getDimensions();

  // State
  // TODO: the panning is false because of not working y condition, though
  // if I fix it, the whole heuristic fails. I am saddness... :(
  var zooming = cameraState.ratio < previousCameraState.ratio,
      panning = cameraState.x !== previousCameraState.x || cameraState.y !== previousCameraState.x,
      unzooming = cameraState.ratio > previousCameraState.ratio,
      unzoomedPanning = !zooming && !unzooming && cameraState.ratio >= 1;

  // Trick to discretize unzooming
  if (unzooming && Math.trunc(cameraState.ratio * 10) % 3 !== 0) return Array.from(displayedLabels);

  // If panning while unzoomed, we shouldn't change label selection
  if (unzoomedPanning && displayedLabels.size !== 0) return Array.from(displayedLabels);

  // Selecting cell
  var baseCell = cameraState.ratio >= 1.3 ? DEFAULT_UNZOOMED_CELL : DEFAULT_CELL;

  // Adapting cell dimensions
  var cellWidthRemainder = dimensions.width % baseCell.width;
  var cellWidth = baseCell.width + cellWidthRemainder / Math.floor(dimensions.width / baseCell.width);

  var cellHeightRemainder = dimensions.height % baseCell.height;
  var cellHeight = baseCell.height + cellHeightRemainder / Math.floor(dimensions.height / baseCell.height);

  // Building the grid
  var grid = {};

  var worthyBuckets = new Set();
  var worthyLabels = [];

  // Selecting worthy labels
  for (var i = 0, l = visibleNodes.length; i < l; i++) {
    var node = visibleNodes[i],
        nodeData = cache[node];

    // When panning, we should not consider nodes that were previously shown
    if (panning && !zooming && !unzooming) {
      if (!displayedLabels.has(node) && previousVisibleNodes.has(node)) continue;
    }

    // Finding our node's cell in the grid
    // TODO: pass dimensions to the function
    var pos = camera.graphToViewport(dimensions, nodeData.x, nodeData.y);

    var xKey = Math.floor(pos.x / cellWidth),
        yKey = Math.floor(pos.y / cellHeight);

    // NOTE: there seems to be overflowing keys but this is actually a good
    // thing since it means we grasp margins.
    var key = xKey + ';' + yKey;

    // When zooming or panning, we aim at keeping the already displayed labels
    if ((zooming || panning && !unzooming) && displayedLabels.has(node)) {
      worthyBuckets.add(key);
      worthyLabels.push(node);
      continue;
    }

    if (worthyBuckets.has(key)) continue;

    // Label resolution
    if (typeof grid[key] === 'undefined') {

      // The cell is empty
      grid[key] = node;
    } else {

      // We must solve a conflict in this cell
      var currentNode = grid[key],
          currentNodeData = cache[currentNode];

      // In case of size equality, we use the node's key so that the
      // process remains deterministic
      if (nodeData.size > currentNodeData.size || nodeData.size === currentNodeData.size && node > currentNode) {
        grid[key] = node;
      }
    }
  }

  // Compiling the labels
  for (var _key in grid) {
    worthyLabels.push(grid[_key]);
  }return worthyLabels;
};